#!/bin/bash

TEMPLATES=templates
TMP=/tmp/
TMPDIR=/tmp/$TEMPLATES
SINGLE=single
MULTI=multi
ALLHOSTDIR=allhost
RMHOST=router_manager
DATAHOSTDIR=datahost
BUNDLE=hce-node-bundle
CFG=$BUNDLE/api/php/cfg
INI=$BUNDLE/api/php/ini

RED='\e[1;31m'
BLUE='\e[1;34m'
DEF='\e[0m'

NODESREG='^[1-9]$|^10$'
HOSTSREG='^[0-9]+$'
OCTET='(25[0-5]|2[0-4][0-9]|[01]?[0-9]?[0-9])'
IPREG='^\$OCTET\.\$OCTET\.\$OCTET\.\$OCTET$'
REPLICAS_POOL_ADMIN_PORTS_N=5530
REPLICAS_POOL_ADMIN_PORTS_M=5630
REPLICAS_POOL_ADMIN_PORTS_R=5730

# FINCTIONS #
function validIp() {
    local  ip=$1
    local  stat=1
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]
    then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

function removeCurrentCfg() {
    read -p "Use local ../cfg/current_cfg.sh or create new? [NEW/use]: " RM
    if [[ ! $RM || $RM == new ]]
    then
        echo new
    elif [[ $RM == use ]]
    then
        echo use
    else
        exit 1
    fi
}

function choiseSystem() {
    read -p "Single or Multihost system [SINGLE/multi]: " SYSTEM
    if [[ ! $SYSTEM ]]
    then
        echo single
    else
        if [[ $SYSTEM == single ]]
        then
            echo single
        elif [[ $SYSTEM == multi ]]
        then
            echo multi
        else
            exit 1
        fi
    fi
}

function tmpPath() {
    read -p "Path for created files (without slash in end): " TMPPATH
    if [[ ! $TMPPATH ]]
    then
        echo /tmp/
    else
        echo $TMPPATH/
    fi
}

function ipOfHosts() {
    read -p "IP for host № "$COUNTER" [127.0.0.1]: " IP
    if [[ ! $IP ]]
    then
        echo 127.0.0.1
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function ipOfRouter() {
    read -p "IP for router ["${IP[1]}"]: " IP
    if [[ ! $IP ]]
    then
        echo ${IP[1]}
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function ipOfManager() {
    read -p "IP for manager ["${IP[1]}"]: " IP
    if [[ ! $IP ]]
    then
        echo ${IP[1]}
    elif validIp $IP
    then
        echo $IP
    else
        exit 1
    fi
}

function numberOfHosts() {
    read -p "Number of hosts [2]-[...]: " HOSTS
    if [[ ! $HOSTS ]]
    then
        echo 2
    elif [[ $HOSTS =~ $HOSTSREG ]]
    then
        echo $HOSTS
    else
        exit 1
    fi
}

function numberOfNodes() {
    # $1 - type of manager
    if
        [[ $1 == 'r' ]]
    then
        read -p "Number of nodes [4]-10: " NODES
        if [[ ! $NODES || $NODES -lt 4 ]]
        then
            echo 4
        elif [[ $NODES =~ $NODESREG ]]
        then
            echo $NODES
        else
            exit 1
        fi
    else
        read -p "Number of nodes [1]-10: " NODES
        if [[ ! $NODES ]]
        then
            echo 1
        elif [[ $NODES =~ $NODESREG ]]
        then
            echo $NODES
        else
            exit 1
        fi
    fi
}

function isData() {
    read -p "Data on this host? [y]/n: " ISDATA
    if [[ ! $ISDATA || $ISDATA == 'y' ]]
    then
        echo Y
    elif [[ $ISDATA == 'n' ]]
    then
        echo N
    else
        exit 1
    fi
}

function typeOfManager() {
    read -p "Type of manager [n]/m/r: " MANAGER_TYPE
    if [[ ! $MANAGER_TYPE ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'n' ]]
    then
        echo n
    elif [[ $MANAGER_TYPE == 'm' ]]
    then
        echo m
    elif [[ $MANAGER_TYPE == 'r' ]]
    then
        echo r
    else
        exit 1
    fi
}

function checkCreateIni() {
    # $1 - type of manager
    if [[ $1 == 'n' ]]
    then
        read -p "Create ini files? [y]/n: " CHECK
        if [[ ! $CHECK ]]
        then
            echo Y
        elif [[ $CHECK == 'y' ]]
        then
            echo Y
        elif [[ $CHECK == 'n' ]]
        then
            echo N
        else
            exit 1
        fi
    elif
        [[ $1 == 'm' ]]
    then
        read -p "Create ini files? y/[n]: " CHECK
        if [[ ! $CHECK ]]
        then
            echo N
        elif [[ $CHECK == 'y' ]]
        then
            echo Y
        elif [[ $CHECK == 'n' ]]
        then
            echo N
        else
            exit 1
        fi
    elif
        [[ $1 == 'r' ]]
    then
        read -p "Create ini files? [y]/n: " CHECK
        if [[ ! $CHECK ]]
        then
            echo Y
        elif [[ $CHECK == 'y' ]]
        then
            echo Y
        elif [[ $CHECK == 'n' ]]
        then
            echo N
        else
            exit 1
        fi
    else
        exit 1
    fi

}

function typeOfCluster() {
    read -p "Type of cluster [smanager]/rmanager/rmanager-rnd: " CLUSTER
    if [[ ! $CLUSTER ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'smanager' ]]
    then
        echo smanager
    elif [[ $CLUSTER == 'rmanager' ]]
    then
        echo rmanager
    elif [[ $CLUSTER == 'rmanager-rnd' ]]
    then
        echo rmanager-rnd
    else
        exit 1
    fi
}

function replicasPoolAdminPorts () {
    # $1 - name of manager, $2 - count of nodes
    for (( COUNT=0; COUNT<$2; COUNT++ ))
    do
        PORT=$[$1+$COUNT]
        if [[ ! $PORTS ]]
        then
            PORTS=$PORT
        else
            PORTS=${PORTS}' '$PORT
        fi
    done
    echo $PORTS
}

function search () {
    # $1 - filename, where search
    # $2 - search string
    while read line
    do
        if [[ $line == $2 ]]
        then
            echo true
            exit 1
        fi
    done < $1
}
# END OF FINCTIONS #

# CHECK ARCHIVE WITH TEMPLATES #
if [[ ! -d $TEMPLATES ]]
then
    echo -e $RED"Directory with templates not found. Aborting"$DEF
    exit
fi
# END OF CHECK ARCHIVE WITH TEMPLATES #

# GET VARIABLES #
clear
if [[ $# -ne 0 ]]
then
# FROM FILES #
    FILE=$1
    if [[ -a $FILE ]]
    then
        while read line
        do
            if [[ $(echo $line | grep \"HOSTS\") ]]
            then
                HOSTS=$(echo $line | awk -F\" '/"HOSTS"/ {print $4}')
            elif [[ $(echo $line | grep \"ROUTER_IP\") ]]
            then
                ROUTER_IP=$(echo $line | awk -F\" '/"ROUTER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"MANAGER_IP\") ]]
            then
                MANAGER_IP=$(echo $line | awk -F\" '/"MANAGER_IP"/ {print $4}')
            elif [[ $(echo $line | grep \"CLUSTER\") ]]
            then
                CLUSTER=$(echo $line | awk -F\" '/"CLUSTER"/ {print $4}')
            elif [[ $(echo $line | grep \"IP\") ]]
            then
                COUNTER=1
                IPS=$(echo $line | grep \"IP\" | awk -F[ '{print $2}' | awk -F] '{print $1}')
                for I in $(echo $IPS | tr "," "\n")
                do
                    IPTMP=$(echo $I | sed "s/\"//g")
                    IP[$COUNTER]=$IPTMP
                    unset IPTMP
                    let COUNTER=COUNTER+1
                done
            fi
        done < $FILE
    fi
else
# FROM STDIN #
    # REMOVE CURRENT CFG OR NOT #
    RMCURCFG=$(removeCurrentCfg)
    if [[ -z $RMCURCFG ]]
    then
        echo -e $RED"Input false. Exit"$DEF
        exit 1
    fi
    # GET OUT PATH #
    OUTDIR=$(tmpPath)

    # SINGLE OR MULTI SYSTEM #
    SYSTEM=$(choiseSystem)
    if [[ -z $SYSTEM ]]
    then
        echo -e $RED"System not correct, select single or multi"$DEF
        exit 1
    fi
    if [[ $SYSTEM == single ]]
    then
    # SINGLE SYSTEM #
        NODES=$(numberOfNodes)
        if [[ -z $NODES ]]
        then
            echo -e $RED"Number of nodes is not valid"$DEF
            exit 1
        fi
        MANAGER_TYPE=$(typeOfManager)
        if [[ -z $MANAGER_TYPE ]]
        then
            echo -e $RED"Manager type is not valid"$DEF
            exit 1
        fi
        CREATE_INI=$(checkCreateIni $MANAGER_TYPE)
        if [[ -z $CREATE_INI ]]
        then
            echo -e $RED"Answer is not valid"$DEF
            exit 1
        fi
    elif [[ $SYSTEM == multi ]]
    then
    # MULTI SYSTEM #
        COUNTER=1
        MANAGER_TYPE=$(typeOfManager)
        if [[ -z $MANAGER_TYPE ]]
        then
            echo -e $RED"Manager type is not valid"$DEF
            exit 1
        fi
        HOSTS=$(numberOfHosts)
        if [[ -z $HOSTS ]]
        then
            echo -e $RED"Number of hosts is not valid"$DEF
            exit 1
        fi
        while [[  $COUNTER -le $HOSTS ]]
        do
            IPTMP=$(ipOfHosts)
            if [[ -z $IPTMP ]]
            then
                echo -e $RED"IP address is not valid"$DEF
                exit 1
            else
                IP[$COUNTER]=$IPTMP
            fi
            ISDATATMP=$(isData)
            if [[ -z $ISDATATMP ]]
            then
                echo -e $RED"Answer is not valid"$DEF
                exit 1
            else
                ISDATA[$COUNTER]=$ISDATATMP
            fi
            if [[ $ISDATATMP == 'Y' ]]
            then
                NODESTMP=$(numberOfNodes $MANAGER_TYPE)
                if [[ -z $NODESTMP ]]
                then
                    echo -e $RED"Number of nodes is not valid"$DEF
                    exit 1
                else
                    NODES[$COUNTER]=$NODESTMP
                fi
            fi
            let COUNTER=COUNTER+1
        done
        ROUTER_IP=$(ipOfRouter)
        if [[ -z $ROUTER_IP ]]
        then
            echo -e $RED"IP address for router is not valid"$DEF
            exit 1
        fi
        MANAGER_IP=$(ipOfManager)
        if [[ -z $MANAGER_IP ]]
        then
            echo -e $RED"IP address for manager is not valid"$DEF
            exit 1
        fi

        CREATE_INI=$(checkCreateIni $MANAGER_TYPE)
        if [[ -z $CREATE_INI ]]
        then
            echo -e $RED"Answer is not valid"$DEF
            exit 1
        fi
        #CLUSTER=$(typeOfCluster)
        #if [[ -z $CLUSTER ]]
        #then
        #    echo -e $RED"Type of cluster is not valid"$DEF
        #    exit 1
        #fi
    else
        exit 1
    fi
fi
# END OF GET VARIABLES #

# CREATE CONFIGS #
# DELETE OLD CONFIGS #
rm -rf $TMPDIR

if [[ $SYSTEM == single ]]
then
# SINGLE SYSTEM #
    N_NAME=localhost_n_cfg.sh
    M_NAME=localhost_m_cfg.sh
    R_NAME=localhost_r_cfg.sh
    mkdir -p $TMPDIR/$SINGLE/$CFG
    mkdir -p $TMPDIR/$SINGLE/$INI
    # CP current_cfg.sh TO TEMPLATES #
    if [[ -f ../cfg/current_cfg.sh && $RMCURCFG == use ]]
    then
        cp ../cfg/current_cfg.sh $TMPDIR/$SINGLE/$CFG
    else
        cp $TEMPLATES/$SINGLE/$CFG/current_cfg.sh $TMPDIR/$SINGLE/$CFG/
    fi
    if [[ $MANAGER_TYPE == n ]]
    then
        # SEARCH IN current_cfg.sh DUBLE CONFIG #
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        # COPY N-CONFIG TO TEMPLATE DIRECTORY #
        cp $TEMPLATES/$SINGLE/$CFG/n_cfg.sh $TMPDIR/$SINGLE/$CFG/$N_NAME
        # PORTS OF NODES #
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N $NODES)
        sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/$SINGLE/$CFG/$N_NAME
        # MAKE node_poolX.ini #
        for (( COUNT=1; COUNT<=$NODES; COUNT++ ))
        do
            if [[ -z $RPORT ]]
            then
                RPORT=$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNT-1))
            else
                RPORT="$RPORT,$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNT-1))"
            fi
            if [[ $CREATE_INI == 'Y' ]]
            then
                cp $TEMPLATES/$SINGLE/$INI/node_poolX.ini $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNT-1))'/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NOTIFICATIONIP_#/localhost/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
            fi
        done
        sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"localhost\""'/' $TMPDIR/$SINGLE/$CFG/$N_NAME
        sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/$SINGLE/$CFG/$N_NAME
    elif [[ $MANAGER_TYPE == m ]]
    then
        # SEARCH IN current_cfg.sh DUBLE CONFIG #
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        # COPY M-CONFIG TO TEMPLATE DIRECTORY #
        cp $TEMPLATES/$SINGLE/$CFG/n_cfg.sh $TMPDIR/$SINGLE/$CFG/$N_NAME
        cp $TEMPLATES/$SINGLE/$CFG/m_cfg.sh $TMPDIR/$SINGLE/$CFG/$M_NAME
        # PORTS OF NODES #
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N $NODES)
        sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/$SINGLE/$CFG/$N_NAME
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M $NODES)
        sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/$SINGLE/$CFG/$M_NAME
        # MAKE node_poolX.ini #
        for (( COUNT=1; COUNT<=$NODES; COUNT++ ))
        do
            if [[ -z $RPORT ]]
            then
                RPORT=$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNT-1))
            else
                RPORT="$RPORT,$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNT-1))"
            fi
            if [[ $CREATE_INI == 'Y' ]]
            then
                cp $TEMPLATES/$SINGLE/$INI/node_poolX.ini $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNT-1))'/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NOTIFICATIONIP_#/localhost/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
                sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/$SINGLE/$INI/node_pool$COUNT.ini
            fi
        done
        sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"localhost\""'/' $TMPDIR/$SINGLE/$CFG/$M_NAME
        sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/$SINGLE/$CFG/$M_NAME
    elif [[ $MANAGER_TYPE == r ]]
    then
        # SEARCH IN current_cfg.sh DUBLE CONFIG #
        DUBLE=$(search $TMPDIR/$SINGLE/$CFG/current_cfg.sh "source ../cfg/$R_NAME")
        if [[ -z $DUBLE ]]
        then
            # ADD TO current_cfg.sh #
            sed -i 's/#_CONFIGFILE_R_#/'"source ..\/cfg\/$R_NAME\n  #_CONFIGFILE_R_#"'/g' $TMPDIR/$SINGLE/$CFG/current_cfg.sh
        fi
        # COPY R-CONFIG TO TEMPLATE DIRECTORY #
        cp $TEMPLATES/$SINGLE/$CFG/r_cfg.sh $TMPDIR/$SINGLE/$CFG/$R_NAME
        # PORTS OF NODES #
        PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_R $NODES)
        sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/$SINGLE/$CFG/$R_NAME
        # MAKE node_pool_rX.ini #
        for (( COUNT=1; COUNT<=$NODES; COUNT++ ))
        do
            if [[ -z $RPORT ]]
            then
                RPORT=$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNT-1))
            else
                RPORT="$RPORT,$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNT-1))"
            fi
            if [[ $CREATE_INI == 'Y' ]]
            then
                cp $TEMPLATES/$SINGLE/$INI/node_pool_rX.ini $TMPDIR/$SINGLE/$INI/node_pool_r$COUNT.ini
                sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/$SINGLE/$INI/node_pool_r$COUNT.ini
                sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNT-1))'/' $TMPDIR/$SINGLE/$INI/node_pool_r$COUNT.ini
                sed -i 's/#_NOTIFICATIONIP_#//' $TMPDIR/$SINGLE/$INI/node_pool_r$COUNT.ini
                sed -i 's/#_NOTIFICATIONPORT_#//' $TMPDIR/$SINGLE/$INI/node_pool_r$COUNT.ini
            fi
        done
        sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"localhost\""'/' $TMPDIR/$SINGLE/$CFG/$R_NAME
        sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/$SINGLE/$CFG/$R_NAME
    else
        echo -e $RED'Fail'$DEF
        exit 1
    fi
# END OF SINGLE SYSTEM #

else
# MULTI SYSTEM #
    mkdir $TMPDIR
    for (( COUNT=1; COUNT<=$HOSTS; COUNT++ ))
    do
        mkdir -p $TMPDIR/${IP[$COUNT]}/$CFG
        mkdir -p $TMPDIR/${IP[$COUNT]}/$INI
        unset RPORT
        unset RHOST
        N_NAME=${IP[$COUNT]}_n_cfg.sh
        M_NAME=${IP[$COUNT]}_m_cfg.sh
        R_NAME=${IP[$COUNT]}_r_cfg.sh
        if [[ ${IP[$COUNT]} == $MANAGER_IP && ${IP[$COUNT]} == $ROUTER_IP && ${ISDATA[$COUNT]} == 'Y' ]]
        then
            # MANAGER + ROUTER + DATA HOST #
            # CP current_cfg.sh TO TEMPLATES #
            if [[ -f ../cfg/current_cfg.sh && $RMCURCFG == use ]]
            then
                cp ../cfg/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            else
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            fi
            if [[ $MANAGER_TYPE == n ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RHOSTTEMP
                    unset RPORTTEMP
                    if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                    then
                        for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                        do
                            if [[ -z $RPORTTEMP ]]
                            then
                                RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))
                            else
                                RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))"
                            fi
                            if [[ -z $RHOSTTEMP ]]
                            then
                                RHOSTTEMP=${IP[$COUNTHOSTS]}
                            else
                                RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                            fi
                        done
                        if [[ -z $RPORT ]]
                        then
                            RPORT=$RPORTTEMP
                        else
                            RPORT="$RPORT,$RPORTTEMP"
                        fi

                        if [[ -z $RHOST ]]
                        then
                            RHOST=$RHOSTTEMP
                        else
                            RHOST="$RHOST,$RHOSTTEMP"
                        fi
                    fi
                done
                sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # MAKE node_poolX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$INI/node_poolX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#/localhost/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                    done
                fi
            elif [[ $MANAGER_TYPE == m ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG AND N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RHOSTTEMP
                    unset RPORTTEMP
                    if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                    then
                        for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                        do
                            if [[ -z $RPORTTEMP ]]
                            then
                                RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))
                            else
                                RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))"
                            fi
                            if [[ -z $RHOSTTEMP ]]
                            then
                                RHOSTTEMP=${IP[$COUNTHOSTS]}
                            else
                                RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                            fi
                        done
                        if [[ -z $RPORT ]]
                        then
                            RPORT=$RPORTTEMP
                        else
                            RPORT="$RPORT,$RPORTTEMP"
                        fi

                        if [[ -z $RHOST ]]
                        then
                            RHOST=$RHOSTTEMP
                        else
                            RHOST="$RHOST,$RHOSTTEMP"
                        fi
                    fi
                done
                sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                unset RPORT
                unset RHOST
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RHOSTTEMP
                    unset RPORTTEMP
                    if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                    then
                        for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                        do
                            if [[ -z $RPORTTEMP ]]
                            then
                                RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))
                            else
                                RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))"
                            fi
                            if [[ -z $RHOSTTEMP ]]
                            then
                                RHOSTTEMP=${IP[$COUNTHOSTS]}
                            else
                                RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                            fi
                        done
                        if [[ -z $RPORT ]]
                        then
                            RPORT=$RPORTTEMP
                        else
                            RPORT="$RPORT,$RPORTTEMP"
                        fi

                        if [[ -z $RHOST ]]
                        then
                            RHOST=$RHOSTTEMP
                        else
                            RHOST="$RHOST,$RHOSTTEMP"
                        fi
                    fi
                done
                sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # MAKE node_poolX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$INI/node_poolX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#/localhost/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                    done
                fi
            elif [[ $MANAGER_TYPE == r ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$R_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_R_#/'"source ..\/cfg\/$R_NAME\n  #_CONFIGFILE_R_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY R-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$CFG/r_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # PORTS OF NODES #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_R ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME

                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RHOSTTEMP
                    unset RPORTTEMP
                    if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                    then
                        for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                        do
                            if [[ -z $RPORTTEMP ]]
                            then
                                RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))
                            else
                                RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))"
                            fi
                            if [[ -z $RHOSTTEMP ]]
                            then
                                RHOSTTEMP=${IP[$COUNTHOSTS]}
                            else
                                RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                            fi
                        done
                        if [[ -z $RPORT ]]
                        then
                            RPORT=$RPORTTEMP
                        else
                            RPORT="$RPORT,$RPORTTEMP"
                        fi

                        if [[ -z $RHOST ]]
                        then
                            RHOST=$RHOSTTEMP
                        else
                            RHOST="$RHOST,$RHOSTTEMP"
                        fi
                    fi
                done
                sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # MAKE node_pool_rX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$ALLHOSTDIR/$INI/node_pool_rX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/localhost/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#//' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#//' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                    done
                fi
            else
                echo -e $RED'Fail'$DEF
                exit 1
            fi
        elif [[ ${IP[$COUNT]} == $MANAGER_IP && ${IP[$COUNT]} != $ROUTER_IP && ${ISDATA[$COUNT]} == 'Y' ]]
        then
            # MANAGER + DATA - ROUTER #
            echo -e $BLUE"manager+data-router"$DEF
        elif [[ ${IP[$COUNT]} != $MANAGER_IP && ${IP[$COUNT]} == $ROUTER_IP && ${ISDATA[$COUNT]} == 'Y' ]]
        then
            # ROUTER + DATA - MANAGER #
            echo -e $BLUE"manager+data-router"$DEF
        elif [[ ${IP[$COUNT]} != $MANAGER_IP && ${IP[$COUNT]} != $ROUTER_IP && ${ISDATA[$COUNT]} == 'Y' ]]
        then
            # DATA ONLY #
            # CP current_cfg.sh TO TEMPLATES #
            if [[ -f ../cfg/current_cfg.sh && $RMCURCFG == use ]]
            then
                cp ../cfg/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            else
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            fi
            if [[ $MANAGER_TYPE == n ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # PORTS OF NODES AND MANAGER #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/__MANAGER__/'"\"$MANAGER_IP\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                sed -i 's/#_RHOST_#/REMOTE_HOSTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/#_RPORT_#/REMOTE_PORTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # NODE_APP_LOG_PREFIX #
                sed -i 's/__NODEAPPLOGPREFIX__/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # MAKE node_poolX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$INI/node_poolX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#/'$MANAGER_IP'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                    done
                fi
            elif [[ $MANAGER_TYPE == m ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG AND M-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # PORTS OF NODES AND MANAGER #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_N ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/__MANAGER__/'"\"$MANAGER_IP\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_M ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                sed -i 's/__MANAGER__/'"\"$MANAGER_IP\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                sed -i 's/#_RHOST_#/REMOTE_HOSTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/#_RPORT_#/REMOTE_PORTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/#_RHOST_#/REMOTE_HOSTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                sed -i 's/#_RPORT_#/REMOTE_PORTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # NODE_APP_LOG_PREFIX #
                sed -i 's/__NODEAPPLOGPREFIX__/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                sed -i 's/__NODEAPPLOGPREFIX__/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # MAKE node_poolX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$INI/node_poolX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#/'$MANAGER_IP'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#/5500/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool$COUNTNODES.ini
                done
                fi
            elif [[ $MANAGER_TYPE == r ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$R_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_R_#/'"source ..\/cfg\/$R_NAME\n  #_CONFIGFILE_R_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY R-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$CFG/r_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # PORTS OF NODES AND MANAGER #
                PORTSLIST=$(replicasPoolAdminPorts REPLICAS_POOL_ADMIN_PORTS_R ${NODES[$COUNT]})
                sed -i 's/__PORTLIST__/'"$PORTSLIST"'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                sed -i 's/__MANAGER__/'"\"$MANAGER_IP\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME

                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                sed -i 's/#_RHOST_#/REMOTE_HOSTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                sed -i 's/#_RPORT_#/REMOTE_PORTS=""/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # NODE_APP_LOG_PREFIX #
                sed -i 's/__NODEAPPLOGPREFIX__/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # MAKE node_poolX.ini #
                if [[ $CREATE_INI == 'Y' ]]
                then
                    for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNT]}; COUNTNODES++ ))
                    do
                        cp $TEMPLATES/$MULTI/$DATAHOSTDIR/$INI/node_pool_rX.ini $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NODE_HOST_#/'${IP[$COUNT]}'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NODE_PORT_#/'$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))'/' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONIP_#//' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                        sed -i 's/#_NOTIFICATIONPORT_#//' $TMPDIR/${IP[$COUNT]}/$INI/node_pool_r$COUNTNODES.ini
                    done
                fi
            else
                echo -e $RED'Fail'$DEF
                exit 1
            fi
        elif [[ ${IP[$COUNT]} == $MANAGER_IP && ${IP[$COUNT]} == $ROUTER_IP && ${ISDATA[$COUNT]} == 'N' ]]
        then
            # ROUTER+MANAGER ONLY #
            # CP current_cfg.sh TO TEMPLATES #
            if [[ -f ../cfg/current_cfg.sh && $RMCURCFG == use ]]
            then
                cp ../cfg/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            else
                cp $TEMPLATES/$MULTI/$RMHOST/$CFG/current_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/
            fi
            if [[ $MANAGER_TYPE == n ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$RMHOST/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RPORT
                    unset RHOST
                    for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                    do
                        unset RHOSTTEMP
                        unset RPORTTEMP
                        if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                        then
                            for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                            do
                                if [[ -z $RPORTTEMP ]]
                                then
                                    RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))
                                else
                                    RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))"
                                fi
                                if [[ -z $RHOSTTEMP ]]
                                then
                                    RHOSTTEMP=${IP[$COUNTHOSTS]}
                                else
                                    RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                                fi
                            done
                            if [[ -z $RPORT ]]
                            then
                                RPORT=$RPORTTEMP
                            else
                                RPORT="$RPORT,$RPORTTEMP"
                            fi

                            if [[ -z $RHOST ]]
                            then
                                RHOST=$RHOSTTEMP
                            else
                                RHOST="$RHOST,$RHOSTTEMP"
                            fi
                        fi
                    done
                    sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                    sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                done
            elif [[ $MANAGER_TYPE == m ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$N_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_N_#/'"source ..\/cfg\/$N_NAME\n  #_CONFIGFILE_N_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$M_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_M_#/'"source ..\/cfg\/$M_NAME\n  #_CONFIGFILE_M_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$RMHOST/$CFG/n_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                cp $TEMPLATES/$MULTI/$RMHOST/$CFG/m_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RPORT
                    unset RHOST
                    for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                    do
                        unset RHOSTTEMP
                        unset RPORTTEMP
                        if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                        then
                            for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                            do
                                if [[ -z $RPORTTEMP ]]
                                then
                                    RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))
                                else
                                    RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_N+$COUNTNODES-1))"
                                fi
                                if [[ -z $RHOSTTEMP ]]
                                then
                                    RHOSTTEMP=${IP[$COUNTHOSTS]}
                                else
                                    RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                                fi
                            done
                            if [[ -z $RPORT ]]
                            then
                                RPORT=$RPORTTEMP
                            else
                                RPORT="$RPORT,$RPORTTEMP"
                            fi

                            if [[ -z $RHOST ]]
                            then
                                RHOST=$RHOSTTEMP
                            else
                                RHOST="$RHOST,$RHOSTTEMP"
                            fi
                        fi
                    done
                    sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                    sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$N_NAME
                    unset RPORT
                    unset RHOST
                    for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                    do
                        unset RHOSTTEMP
                        unset RPORTTEMP
                        if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                        then
                            for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                            do
                                if [[ -z $RPORTTEMP ]]
                                then
                                    RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))
                                else
                                    RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_M+$COUNTNODES-1))"
                                fi
                                if [[ -z $RHOSTTEMP ]]
                                then
                                    RHOSTTEMP=${IP[$COUNTHOSTS]}
                                else
                                    RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                                fi
                            done
                            if [[ -z $RPORT ]]
                            then
                                RPORT=$RPORTTEMP
                            else
                                RPORT="$RPORT,$RPORTTEMP"
                            fi

                            if [[ -z $RHOST ]]
                            then
                                RHOST=$RHOSTTEMP
                            else
                                RHOST="$RHOST,$RHOSTTEMP"
                            fi
                        fi
                    done
                    sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                    sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$M_NAME
                done
            elif [[ $MANAGER_TYPE == r ]]
            then
                # SEARCH IN current_cfg.sh DUBLE CONFIG #
                DUBLE=$(search $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh "source ../cfg/$R_NAME")
                if [[ -z $DUBLE ]]
                then
                    # ADD TO current_cfg.sh #
                    sed -i 's/#_CONFIGFILE_R_#/'"source ..\/cfg\/$R_NAME\n  #_CONFIGFILE_R_#"'/g' $TMPDIR/${IP[$COUNT]}/$CFG/current_cfg.sh
                fi
                # COPY N-CONFIG TO TEMPLATE DIRECTORY #
                cp $TEMPLATES/$MULTI/$RMHOST/$CFG/r_cfg.sh $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                # CREATE RHOST AND RPORT FOR X_cfg.sh #
                for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                do
                    unset RPORT
                    unset RHOST
                    for (( COUNTHOSTS=1; COUNTHOSTS<=$HOSTS; COUNTHOSTS++ ))
                    do
                        unset RHOSTTEMP
                        unset RPORTTEMP
                        if [[ ${IP[$COUNTHOSTS]} != $MANAGER_IP ]]
                        then
                            for (( COUNTNODES=1; COUNTNODES<=${NODES[$COUNTHOSTS]}; COUNTNODES++ ))
                            do
                                if [[ -z $RPORTTEMP ]]
                                then
                                    RPORTTEMP=$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))
                                else
                                    RPORTTEMP="$RPORTTEMP,$(($REPLICAS_POOL_ADMIN_PORTS_R+$COUNTNODES-1))"
                                fi
                                if [[ -z $RHOSTTEMP ]]
                                then
                                    RHOSTTEMP=${IP[$COUNTHOSTS]}
                                else
                                    RHOSTTEMP="$RHOSTTEMP,${IP[$COUNTHOSTS]}"
                                fi
                            done
                            if [[ -z $RPORT ]]
                            then
                                RPORT=$RPORTTEMP
                            else
                                RPORT="$RPORT,$RPORTTEMP"
                            fi

                            if [[ -z $RHOST ]]
                            then
                                RHOST=$RHOSTTEMP
                            else
                                RHOST="$RHOST,$RHOSTTEMP"
                            fi
                        fi
                    done
                    sed -i 's/#_RHOST_#/'"REMOTE_HOSTS=\"$RHOST\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                    sed -i 's/#_RPORT_#/'"REMOTE_PORTS=\"$RPORT\""'/' $TMPDIR/${IP[$COUNT]}/$CFG/$R_NAME
                done
            else
                echo -e $RED'Fail'$DEF
                exit 1
            fi
        else
            # ALL IN SINGLE HOST #
            echo -e $BLUE"all in single host"$DEF
        fi
    done
fi

# MV 2 OUTDIR #
if [[ $TMP != $OUTDIR ]]
then
    if [[ -d $OUTDIR ]]
    then
        rm -rf $OUTDIR/*
    else
        mkdir -p $OUTDIR
    fi
    mv $TMPDIR/* $OUTDIR
    rmdir $TMPDIR
fi

echo -e $BLUE'Complete'$DEF