'''
@package: dc
@author igor
@link: http://hierarchical-cluster-engine.com/
@copyright: Copyright &copy; 2013-2014 IOIX Ukraine
@license: http://hierarchical-cluster-engine.com/license/
@since: 0.1
'''
import logging
import Constants
import time
import dc.EventObjects
import app.SQLCriterions
from Constants import LOGGER_NAME

logger = logging.getLogger(LOGGER_NAME)

# #Class contains operations common for all tasks
class BaseTask(object):

  # #check if given site exist in current db
  #
  # @param siteId id of checking site
  # @param queryCallback function for queries execution
  # @return True if exist, or False
  def isSiteExist(self, siteId, queryCallback, userId=None):
    retVal = False
    if siteId is not None:
      query = Constants.CHECK_TABLE_SQL_TEMPLATE % siteId
      if userId is not None:
        query += (Constants.CHECK_TABLE_SQL_ADDITION % str(userId))
      res = queryCallback(query, Constants.PRIMARY_DB_ID)
      if res is not None and len(res) > 0 and len(res[0]) > 0 and res[0][0] > 0:
        retVal = True# first elem from first tuple

    return retVal


  # #Generates criterion string from urlFetch.criterions or urlUpdate.criterions fields
  #
  # @param criterions - incoming criterions
  # @param additionWhere - additions WHERE cause
  def generateCriterionSQL(self, criterions, additionWhere=None, siteId=None):
    return app.SQLCriterions.generateCriterionSQL(criterions, additionWhere, siteId)
    '''
    additionString = ""
    #Criterions "WHERE" block
    if dc.EventObjects.URLFetch.CRITERION_WHERE in criterions and \
    criterions[dc.EventObjects.URLFetch.CRITERION_WHERE] is not None:
      additionString += (" " + dc.EventObjects.URLFetch.CRITERION_WHERE + " ")
      if additionWhere is not None:
        additionString += (additionWhere + " AND ")
      additionString += str(criterions[dc.EventObjects.URLFetch.CRITERION_WHERE])
    elif additionWhere is not None:
      additionString += (" " + dc.EventObjects.URLFetch.CRITERION_WHERE + " ")
      additionString += additionWhere
    #Criterions "ORDER" block
    if dc.EventObjects.URLFetch.CRITERION_ORDER in criterions and \
    criterions[dc.EventObjects.URLFetch.CRITERION_ORDER] is not None:
      additionString += (" " + dc.EventObjects.URLFetch.CRITERION_ORDER + " ")
      additionString += str(criterions[dc.EventObjects.URLFetch.CRITERION_ORDER])
    #Criterions "LIMIT" block
    if dc.EventObjects.URLFetch.CRITERION_LIMIT in criterions and \
    criterions[dc.EventObjects.URLFetch.CRITERION_LIMIT] is not None:
      localLimit = criterions[dc.EventObjects.URLFetch.CRITERION_LIMIT]
      if type(localLimit) == type([]):
        localString = ("%s , %s" % (str(localLimit[0]), str(localLimit[1])))
      else:
        localString = str(localLimit)
      additionString += (" " + dc.EventObjects.URLFetch.CRITERION_LIMIT + " " + localString)
    return additionString
    '''


  ##method returns ids elements fetching by criterions
  #
  #@param criterions - incoming criterions
  #@param queryCallback function for queries execution
  #@return site ids
  def fetchByCriterions(self, criterions, queryCallback):
    ids = []
    additionWhere = self.generateCriterionSQL(criterions)
    if additionWhere is not None and additionWhere != "":
      query = "SELECT `id` FROM `sites`" + additionWhere
      res = queryCallback(query, Constants.PRIMARY_DB_ID)
      if hasattr(res, '__iter__'):
        for elem in res:
          ids.append(elem[0])
    return ids


  # #Cross-app locking, based on MySQL db, locking
  #
  # @param mutexName - mutex name
  # @param queryCallback queryCallback function for queries execution
  # @param sleepTime timeout (in seconds) between lock tries
  # @param timeout - internal lock timeout
  def dbLock(self, mutexName, queryCallback, sleepTime=1, timeout=0):
    logger.debug(">>> BaseTask Class. Lock start name=" + mutexName)
    LOCK_QUERY_TEMPLATE = "SELECT mutexLock('%s', %s)"
    query = LOCK_QUERY_TEMPLATE % (mutexName, str(Constants.DB_LOCK_APPLICATION_ID))
    res = queryCallback(query, Constants.PRIMARY_DB_ID)
    while (res is not None and len(res) > 0 and res[0][0] == 0):
      time.sleep(sleepTime)
      res = queryCallback(query, Constants.PRIMARY_DB_ID)
    logger.debug(">>> BaseTask Class. Lock finish name=" + mutexName)


  # #Cross-app locking, based on MySQL db, unlocking
  #
  # @param mutexName - mutex name
  # @param queryCallback queryCallback function for queries execution
  def dbUnlock(self, mutexName, queryCallback):
    logger.debug(">>> BaseTask Class. Unlock start name=" + mutexName)
    LOCK_QUERY_TEMPLATE = "SELECT mutexUnlock('%s', %s)"
    query = LOCK_QUERY_TEMPLATE % (mutexName, str(Constants.DB_LOCK_APPLICATION_ID))
    queryCallback(query, Constants.PRIMARY_DB_ID)
    logger.debug(">>> BaseTask Class. Unlock finish name=" + mutexName)

